/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.task;

//Package dependencies
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * This class redirect output and error output of a process.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class Redirector
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The Process to redirect. */
    private Process process_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param process - The process to redirect.
     */
    public Redirector(Process process)
    {
        process_ = process;
    }
    
    // ==================================================================
    //
    // Inner classes.
    //
    // ==================================================================
    
    /** 
     * Redirect the standard output.  
     */
    private class OutputRedirectorThread
       implements Runnable
    {
        /** The input stream to redirect. */
        private InputStream inputstream_;

        /**
         * The default constructor
         * 
         * @param inputstream - The input stream to redirect.
         */
        public 
        OutputRedirectorThread(InputStream inputstream)
        {
            inputstream_ = inputstream;
        }
        
        /**
         * Run the output redirection.
         */
        public void
        run()
        {
            BufferedReader bufferedreader = null;
            String line = null;
            
            try
            {
                // put a BufferedReader on the process output
                bufferedreader =
                    new BufferedReader( new InputStreamReader(inputstream_) );
    
                // read the process output
                while ( (line = bufferedreader.readLine()) != null)
                {
                    System.out.println(line);
                }
            }catch(java.io.IOException e){
                e.printStackTrace();
            }
        }
    }

    /** 
     * Redirect the standard error.
     */
    private class ErrorRedirectorThread
       implements Runnable
    {
        /** The input stream to redirect. */
        private InputStream inputstream_;

        /**
         * The default constructor
         * 
         * @param inputstream - The input stream to redirect.
         */
        public 
        ErrorRedirectorThread(InputStream inputstream)
        {
            inputstream_ = inputstream;
        }
        
        /**
         * Run the output redirection.
         */
        public void
        run()
        {
            BufferedReader bufferedreader = null;
            String line = null;

            try
            {
                // put a BufferedReader on the process output
                bufferedreader =
                    new BufferedReader( new InputStreamReader(inputstream_) );
    
                // read the process output
                while ( (line = bufferedreader.readLine()) != null)
                {
                    System.err.println(line);
                }
            }catch(java.io.IOException e){
                e.printStackTrace();
            }
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
   
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Redirect standard output and standard error to
     * standard output and standard error of the current Thread.
     */
    public void
    redirectOutput()
    {
        Thread t1 = null,
               t2 = null;
        OutputRedirectorThread outThread = null;
        ErrorRedirectorThread errThread = null;
               
               
        // Redirect Output Stream
        try
        {
            outThread = new OutputRedirectorThread(process_.getInputStream());
            errThread = new ErrorRedirectorThread(process_.getErrorStream());
            
            t1 = new Thread( outThread );
            t2 = new Thread( errThread );

            t1.start();
            t2.start();            
        }catch (Exception e){
            e.printStackTrace();
        }
    }

}
