/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlBoolean represents an CORBA IDL boolean value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlBoolean object has  no child nodes.
 * Valid values are TRUE and FALSE.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlBoolean extends IdlNode implements IdlWritable {   
    protected IdlBoolean() {
        setUserObject(this);
        type = "boolean";
        value = "FALSE";
        tc = orb.get_primitive_tc(TCKind.tk_boolean);
    }

    protected IdlBoolean(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlBoolean(TypeCode tc) {
        this();
        setNode(tc);
    }
    
    protected void setNode(Any any) {
        try {    
            setNode(any.type());    
            value = ("" + any.create_input_stream().read_boolean()).toUpperCase();
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            any.type(tc);
            out.write_boolean(value.equals("TRUE") ? true : false);
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            os.write_boolean(value.equals("TRUE") ? true : false);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        value = ("" + is.read_boolean()).toUpperCase();
    }

    // XML section
    
    /**
     * Creates new IdlBoolean object from an XML string.
     *
     * XML format example:
     * <pre>
     *   &lt;boolean&gt;TRUE&lt;/boolean&gt;
     * </pre>
     *
     * @param xml The XML string from which to create a ne IdlBoolean instance.
     */
    public IdlBoolean(String xml) {
        this(XmlNode.getNode(xml));
    }
        
    IdlBoolean(Node n) {
        this();
                       
        if (n == null || !n.getNodeName().toUpperCase().equals("BOOLEAN")) {
            throw new RuntimeException("boolean expected");
        }
        setValue(XmlNode.getText(n).toUpperCase());
    }
    
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        w.write_boolean(value);
    }
}