/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import javax.swing.tree.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * IdlNode is the abstract base class for all concrete IdlNode types.
 * The classes in the idltree library are used to represent arbitrary complex Idl type values as a tree structure.
 * For example an Idl struct such as:
 * <code>
 * <pre>
 *   struct MyStruct {
 *    string member1;
 *    short member2;
 *   };
 * </pre>
 * </code>
 * is represented as an IdlStruct node with two chidren, one of type IdlString and one of type IdlShort.
 * <pre>
 *
 *        IdlStruct
 *           /\
 *          /  \
 *         /    \
 *        /      \
 * IdlString     IdlShort
 * field=member1 field=member2
 * value=""      value=0
 *</pre>
 * The field member variable is used to hold the struct member name.
 * Each IdlNode which is a leaf has a value member variable to hold the current value of the node. The
 * value is maintained as a String but can be converted to a CORBA any value.
 * The tree structure can be constructed in several ways:
 * <BL>
 * <LI>From the content of a CORBA Any value.</LI>
 * <LI>From a CORBA TypeCode value.</LI>
 * <LI>From a XML string representation.</LI>
 * </BL>
 * In reverse, IdlNodes can be converted to CORBA Any and/or an XML string containing all fields and values.
 * Value variables have proper default values. For example when a IdlString node is created, its default value is an empty string.
 * Complex nested Idl structures can be navigated by their field values.
 * An IdlNode tree is derived from a javax.swing.tree.DefaultMutableTreeNode and can be easily drawn as a javax.swing.JTree or
 * interactively edited using a tree editor such as implemented by the IdlEditor class.
 * The tree can also be navigated and controlled through this API.
 */

public abstract class IdlNode extends DefaultMutableTreeNode {
    // In order to save the tree structure as serialized data non
    // serializable fields are transient.
    protected static transient ORB orb = null;
    // The CORBA type code associated with the IDL node.
    protected transient TypeCode tc;
    // Indicator if this node is an end leaf.
    protected boolean isLeaf = true;
    // Only mutable nodes values can be edited with an editor.
    protected boolean isMutable = true;
    // Field is used to add additional information to name a node.
    // For example if a node represents a member of a struct or union, the field variable will hold the
    // member name. If a node is an element in an arrays or sequences, the field variable will hold the index number.
    protected String field = "";
    // The IDL type name of a node.
    protected String type = "";
    // The CORBA repository ID
    protected String id = "";
    // The content of the node represented as a string.
    protected String value = "";

    static {
        try {
            if ( orb == null )
            {
                orb = org.objectweb.ccm.CORBA.TheORB.getORB();
            }
        }
        catch (Error e) {
        }
    }

    protected IdlNode() {
    }

    public static void setOrb(ORB ob) {
        orb = ob;
    }

    /**
     * Sets the TypeCode for this node.
     *
     * @param tc The TypeCode value to set.
     */
    protected void setNode(TypeCode tc) {
        this.tc = tc;
        if (tc.kind().value() == TCKind._tk_alias) {
            try {
                type = tc.id();
                id = tc.id();
            } catch (Exception ex) {
            }
        }
    }

    /**
     * Factory method to create a new IdlNode instance from a TypeCode value.
     *
     * @param tc The TypeCode value from which to create a new instance.
     *
     * @return A new IdlNode instance for the given TypeCode.
     */
    public static IdlNode create(TypeCode tc) {
        try {
            switch (tc.kind().value()) {
                case TCKind._tk_struct: {
                    return new IdlStruct(tc);
                }
                case TCKind._tk_except: {
                    return new IdlException(tc);
                }
                case TCKind._tk_union: {
                    return new IdlUnion(tc);
                }
                case TCKind._tk_array: {
                    return new IdlArray(tc);
                }
                case TCKind._tk_sequence: {
                    return new IdlSequence(tc);
                }
                case TCKind._tk_enum: {
                    return new IdlEnum(tc);
                }
                case TCKind._tk_any: {
                    return new IdlAny(tc);
                }
                case TCKind._tk_octet: {
                    return new IdlOctet(tc);
                }
                case TCKind._tk_short: {
                    return new IdlShort(tc);
                }
                case TCKind._tk_ushort: {
                    return new IdlUshort(tc);
                }
                case TCKind._tk_long: {
                    return new IdlLong(tc);
                }
                case TCKind._tk_ulong: {
                    return new IdlUlong(tc);
                }
                case TCKind._tk_longlong: {
                    return new IdlLonglong(tc);
                }
                case TCKind._tk_ulonglong: {
                    return new IdlUlonglong(tc);
                }
                case TCKind._tk_float: {
                    return new IdlFloat(tc);
                }
                case TCKind._tk_double: {
                    return new IdlDouble(tc);
                }
                case TCKind._tk_longdouble: {
                    return new IdlLongdouble(tc);
                }
                case TCKind._tk_boolean: {
                    return new IdlBoolean(tc);
                }
                case TCKind._tk_char: {
                    return new IdlChar(tc);
                }
                case TCKind._tk_wchar: {
                    return new IdlWchar(tc);
                }
                case TCKind._tk_objref: {
                    return new IdlObject(tc);
                }
                case TCKind._tk_value: {
                    return new IdlValue(tc);
                }
                case TCKind._tk_string: {
                    return new IdlString(tc);
                }
                case TCKind._tk_wstring: {
                    return new IdlWstring(tc);
                }
                case TCKind._tk_fixed: {
                    return new IdlFixed(tc);
                }
                case TCKind._tk_alias: {
                    return create(tc.content_type(), tc);
                }
                default: {
                    throw new RuntimeException("Unexpected type: " + tc.kind().value());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Factory method to create a new IdlNode instance from an aliased TypeCode value.
     *
     * @param alias The original TypeCode value.
     * @param tc The alias content TypeCode value from which to create a new instance.
     *
     * @return A new IdlNode instance for the given TypeCode.
     */
    private static IdlNode create(TypeCode alias, TypeCode tc) {
        try {
            switch (alias.kind().value()) {
                case TCKind._tk_struct: {
                    return new IdlStruct(tc);
                }
                case TCKind._tk_except: {
                    return new IdlException(tc);
                }
                case TCKind._tk_union: {
                    return new IdlUnion(tc);
                }
                case TCKind._tk_array: {
                    return new IdlArray(tc);
                }
                case TCKind._tk_sequence: {
                    return new IdlSequence(tc);
                }
                case TCKind._tk_enum: {
                    return new IdlEnum(tc);
                }
                case TCKind._tk_any: {
                    return new IdlAny(tc);
                }
                case TCKind._tk_octet: {
                    return new IdlOctet(tc);
                }
                case TCKind._tk_short: {
                    return new IdlShort(tc);
                }
                case TCKind._tk_ushort: {
                    return new IdlUshort(tc);
                }
                case TCKind._tk_long: {
                    return new IdlLong(tc);
                }
                case TCKind._tk_ulong: {
                    return new IdlUlong(tc);
                }
                case TCKind._tk_longlong: {
                    return new IdlLonglong(tc);
                }
                case TCKind._tk_ulonglong: {
                    return new IdlUlonglong(tc);
                }
                case TCKind._tk_float: {
                    return new IdlFloat(tc);
                }
                case TCKind._tk_double: {
                    return new IdlDouble(tc);
                }
                case TCKind._tk_longdouble: {
                    return new IdlLongdouble(tc);
                }
                case TCKind._tk_boolean: {
                    return new IdlBoolean(tc);
                }
                case TCKind._tk_char: {
                    return new IdlChar(tc);
                }
                case TCKind._tk_wchar: {
                    return new IdlWchar(tc);
                }
                case TCKind._tk_objref: {
                    return new IdlObject(tc);
                }
                case TCKind._tk_value: {
                    return new IdlValue(tc);
                }
                case TCKind._tk_string: {
                    return new IdlString(tc);
                }
                case TCKind._tk_wstring: {
                    return new IdlWstring(tc);
                }
                case TCKind._tk_fixed: {
                    return new IdlFixed(tc);
                }
                case TCKind._tk_alias: {
                    return create(alias.content_type(), tc);
                }
                default: {
                    throw new RuntimeException("Unexpected type: " + tc.kind().value());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Factory method to create a new IdlNode instance from the content of an Any value.
     *
     * @param any The Any value from which to create a new instance.
     *
     * @return A new IdlNode instance for the given Any.
     */
    public static IdlNode create(Any any) {
        return create(any.type(), any);
    }

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
    }

    /**
     * Factory method to create a new IdlNode instance from an aliased Any value.
     *
     * @param tc The content TypeCode value.
     * @param any The any value from which to create a new instance.
     *
     * @return A new IdlNode instance for the given TypeCode.
     */
    private static IdlNode create(TypeCode tc, Any any) {
        try {
            switch (tc.kind().value()) {
                case TCKind._tk_struct: {
                    return new IdlStruct(any);
                }
                case TCKind._tk_except: {
                    return new IdlException(any);
                }
                case TCKind._tk_union: {
                    return new IdlUnion(any);
                }
                case TCKind._tk_any: {
                    return new IdlAny(any);
                }
                case TCKind._tk_sequence: {
                    return new IdlSequence(any);
                }
                case TCKind._tk_array: {
                    return new IdlArray(any);
                }
                case TCKind._tk_enum: {
                    return new IdlEnum(any);
                }
                case TCKind._tk_short: {
                    return new IdlShort(any);
                }
                case TCKind._tk_ushort: {
                    return new IdlUshort(any);
                }
                case TCKind._tk_char: {
                    return new IdlChar(any);
                }
                case TCKind._tk_wchar: {
                    return new IdlWchar(any);
                }
                case TCKind._tk_string: {
                    return new IdlString(any);
                }
                case TCKind._tk_wstring: {
                    return new IdlWstring(any);
                }
                 case TCKind._tk_long: {
                    return new IdlLong(any);
                }
                case TCKind._tk_ulong: {
                    return new IdlUlong(any);
                }
                case TCKind._tk_longlong: {
                    return new IdlLonglong(any);
                }
                case TCKind._tk_ulonglong: {
                    return new IdlUlonglong(any);
                }
                case TCKind._tk_octet: {
                    return new IdlOctet(any);
                }
                case TCKind._tk_longdouble: {
                    return new IdlLongdouble(any);
                }
                case TCKind._tk_objref: {
                    return new IdlObject(any);
                }
                case TCKind._tk_value: {
                    return new IdlValue(any);
                }
                case TCKind._tk_float: {
                    return new IdlFloat(any);
                }
                case TCKind._tk_double: {
                    return new IdlDouble(any);
                }
                case TCKind._tk_boolean: {
                    return new IdlBoolean(any);
                }
                case TCKind._tk_fixed: {
                    return new IdlFixed(any);
                }
                case TCKind._tk_alias: {
                    return create(tc.content_type(), any);
                }
                default: {
                    throw new RuntimeException("Unexpected type: " + any.type().kind());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Returns a String representation of the node.
     *
     * @return A String representation of the node.
     */
    public String toString() {
        if (field.equals("")) {
            return (type + " " + value).trim();
        } else {
            return (field + " " + type + " " + value).trim();
        }
    }

    /**
     * Returns the child IdlNode which matches the given field path.
     *
     * Search through the node tree for a node that maches the
     * node path. The node path is in the form a.b.c where a, b and c are
     * the field names of child nodes.
     * If no matching path is found, null is returned.
     * This method can be used for example from a scriping environment to
     * retrieve and manipulate IdlNodes.
     *
     * @param path The field path used for the search.
     *
     * @return The child IdlNode for the given field path or null.
     */
    public IdlNode getNode(String path) {
        if (path == null || path.equals("")) {
            return this;
        }
        String current = path;
        int idx = path.indexOf(".");
        if (idx > 0) {
            // set the current field and subtract the current field from the path.
            current = path.substring(0, idx);
            path = path.substring(idx + 1);
        } else {
            // the end of path is reached.
            path = null;
        }
        int cc = getChildCount();
        for (int i = 0; i < cc; i++) {
            IdlNode n = (IdlNode)getChildAt(i);
            if (n instanceof IdlParameter) {
                n = ((IdlParameter)n).getParameter();
            }
            // strip brackets from arrays and sequences
            String f = (n.field.replace('[', ' ').replace(']', ' ')).trim();
            if (f.equals(current)) {
                if (path != null) {
                    // the current fiels matched, continue with the next field.
                    return n.getNode(path);
                } else {
                    // the current field maches and this is the last field
                    return n;
                }
            }
        }
        throw new RuntimeException("No such field '" + current + "' in node type: " + type);
    }

    /**
     * Returns a String with all field names of all child nodes.
     *
     * The field names are separated by a newline character.
     * If the values flag is true, the node value is added to the field name, separated by a space.
     *
     * @param values Boolean flag to indicate if node values must be added to the output.
     *
     * @return A String with field names (with values if requested) for all child nodes.
     */
    public String printFields(boolean values) {
        StringBuffer sb = new StringBuffer();

        for(Enumeration e = depthFirstEnumeration(); e.hasMoreElements();) {
            IdlNode n = (IdlNode)e.nextElement();
            if (!(n instanceof IdlParameter) && n != this) {
                TreeNode[] nn = n.getPath();
                String path = "";
                String dot = "";
                for(int i = getLevel() + 1; i < nn.length; i++) {
                    if (!(nn[i] instanceof IdlParameter)) {
                        path += dot + (((IdlNode)nn[i]).field.replace('[', ' ').replace(']', ' ')).trim();
                        dot = ".";
                    }
                }
                if (values) {
                    sb.append(path + ":" + n.getValue() + "\n");
                } else {
                    sb.append(path + "\n");
                }
            }
        }

        return sb.toString();
    }

    /**
     * Returns a String with all node values for all child nodes.
     *
     * The values are separated by a newline character.
     *
     * @return A String with values for all child nodes.
     */
    public String printValues() {
        StringBuffer sb = new StringBuffer();

        for(Enumeration e =depthFirstEnumeration(); e.hasMoreElements();) {
            IdlNode n = (IdlNode)e.nextElement();
            if (!(n instanceof IdlParameter) && n != this) {
                sb.append(n.getValue() + "\n");
            }
        }

        return sb.toString();
    }

    public void setField(String f) {
        field = f;
    }

    /**
     * Replace it self with the parent with the new node.
     *
     * This method is used to replace a whole subtree of the IdlNode tree.
     *
     * @param n the root of the subtree node which replaces the current node.
     */
    public void replace(IdlNode n) {
        // replace only if type and id match
        if (n.getType().equals(getType()) && n.getId().equals(getId())) {
            n.field = field;
            IdlNode p = (IdlNode)getParent();
            if (p != null) {
                int idx = p.getIndex(this);
                if (idx >= 0) {
                    p.insert(n, idx);
                    return;
                }
                throw new RuntimeException("No such child node.");
            } else {
                throw new RuntimeException("Can't replace root in IdlNode.");
            }
        } else {
            throw new RuntimeException("Unmatching types for IdlNode replacement. Expected: " + getType());
        }
    }

    /**
     * Sets the node value.
     *
     * @param v The assigned value.
     */
    public void setValue(String v) {
        value = v;
    }

    /**
     * Returns the node type string.
     *
     * @return The node type.
     */
    public String getType() {
        return type;
    }

    /**
     * Returns the node value.
     *
     * @return The node value.
     */
    public String getValue() {
        return value;
    }

    /**
     * Returns the repository id for the node.
     *
     * @return The repositiry id.
     */
    public String getId() {
        return id;
    }

    /**
     * Returns the CORBA TypeCode for the node.
     *
     * @return The TypeCode.
     */
    public TypeCode getTypeCode() {
        return tc;
    }


    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        return toDynAny().to_any();
    }

    public org.omg.DynamicAny.DynAny toDynAny() {
        throw new RuntimeException("Illegal operation on this nodetype: " + this.getClass().getName());
    }

    /**
     * Returns true if this node allows subnodes.
     *
     * @return true if this node allows subnodes
     */
    public boolean getAllowsChildren() {
        // overwrite DefaultMutableTreeNode methods
        return !isLeaf;
    }

    /**
     * Returns true if this node is a leaf node.
     *
     * @return true if this node is a leaf node
     */
    public boolean isLeaf() {
        return isLeaf;
    }

    /**
     * Returns true if a value can be assigned to this node.
     *
     * @return true if a value can be assigned to this node
     */
    public boolean isMutable() {
        return isMutable;
    }

    /**
     * Makes a copy of the current node.
     *
     * @return a copy of this node
     */
    public Object clone() {
        try {
            IdlNode t = (IdlNode)super.clone();
            t.parent = null;
            t.children = null;
            t.userObject = t;
            // clone all children
            return t;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Makes a copy of the current node.
     *
     * @return a copy of this node
     */
    public IdlNode copy() {
        return (IdlNode)clone();
    }

    /**
     * Determine if type t is allowed to be pasted into
     * this type.
     *
     * @return true if the types are compatible
     **/
    public boolean canPaste(IdlNode t) {
        return false;
    }

    /**
     * Copies values from the the given node.
     *
     * @param t - the node to copy values from
     */
    void paste(IdlNode t) {
        // initialize all relevant fields
        type = t.type;
        field = t.field;
        children = null;
        allowsChildren = t.allowsChildren;
        userObject = this;

        // copy the value and children
    }

    static protected String[] splitName(String name, String separator) {
        if (name == null) {
            return new String[0];
        }
        StringTokenizer st = new StringTokenizer(name, separator);
        Vector v = new Vector();
        if (name.startsWith(separator)) {
            v.addElement("");
        }
        while (st.hasMoreTokens()) {
            v.addElement(st.nextToken());
        }
        String[] s = new String[v.size()];
        v.copyInto(s);
        return s;
    }
}