package org.coach.idltree;

import org.w3c.dom.Node;
import org.omg.DynamicAny.*;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import org.coach.tracing.api.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * The class IdlOperation represents an operation on an CORBA IDL interface. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlOperation object has child nodes for each in and inout parameter.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlOperation extends IdlNode implements IdlWritable {
    private String[] allTypes;
    private String[] allNames;
    private String[] allDirections;
    private String[] directions;

    protected String[] names = new String[0];
    protected String[] exceptions = new String[0];
    protected Parameter[] params = new Parameter[0];
    protected String[] typeNames = new String[0];
    protected transient TypeCode[] types = new TypeCode[0];
    protected transient TypeCode[] exceptionTypes = new TypeCode[0];

    protected String name;
    protected boolean isOneway;

    protected IdlOperation() {
        isLeaf = false;
        isMutable = false;
        setUserObject(this);
        type = "operation";
    }

    /**
     * Constructs an IdlOperation node.
     *
     * The IdlOperertion node is initialized with IdlNode parameter children from the values array.
     *
     * @param id The interface repository id for the interface to which this operation belongs.
     * @param name The operation name.
     * @param p The array with parameter name, parameter type and direction information.
     * @param values The any values used to initialize the operation parameters.
     * @param isOneway Flag to indicate if this is a oneway operation.
     */
    public IdlOperation(String id, String name, Parameter[] p, org.omg.CORBA.Any[] values, boolean isOneway) {
        this();
        try {
            this.isOneway = isOneway;
            this.id = id;
            this.name = name;
            if (p != null) {
                params = p;
                names = new String[p.length];
                for (int i = 0; i < p.length; i++) {
                    names[i] = p[i].name;
                    add(new IdlParameter(p[i].dir, p[i].name, values[i]));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Return the name of the operation this IdlOperation object represents.
     *
     * @return The operation name.
     */
    public String getName() {
        return name;
    }

    /**
     * Return the list of parameter names for this operation.
     *
     * @return The list of parameter names.
     */
    public String[] getParameterNames() {
        return names;
    }

    /**
     * Return the IdlNode child for a named parameter.
     *
     * @param name The parameter name for which to return the child IdlNode.
     *
     * @return The IdlNode child for the parameter indicated name.
     */
    public IdlNode getParameter(String name) {
        for (int i = 0; i < names.length; i++) {
            if (names[i].equals(name)) {
                return ((IdlParameter)getChildAt(i)).getParameter();
            }
        }
        return null;
    }

    /**
     * Return all IdlNode child nodes.
     *
     * @return An IdlNode array containing all in and inout parameter values.
     */
    public IdlNode[] getParameters() {
        IdlNode[] members = new IdlNode[getChildCount()];
        for(int i = 0; i < members.length; i++) {
            members[i] = ((IdlParameter)getChildAt(i)).getParameter();
        }
        return members;
    }

    /**
     * Returns a string representation for this IdlOperation node.
     *
     * @return The string representation for this IdlOperation node.
     */
    public String toString() {
        return "operation " + id + " " + name;
    }

    /**
     * Fills the given any array with parameter values.
     *
     * The content of the values array is filled with the parameter
     * child nodes converted to any values. The given array length must
     * match the number of in and inout parameter values for this operation.
     */
    public void getValues(org.omg.CORBA.Any[] values) {
        if (values.length != getChildCount()) {
            System.out.println("Invalid array length for any values");
            return;
        }
        for(int i = 0; i < getChildCount(); i++) {
            values[i] = ((IdlParameter)getChildAt(i)).getParameter().toAny();
        }
    }

    // XML section

    /**
     * Create an IdlOperation from an Xml representation.
     *
     * The XML format example
     * <pre>
     * &lt;operation id="IDL:example/MyInterface:1.0"&gt;
     *  &lt;in name="p1"&gt;
     *   &lt;string&gt;my parameter value&lt;/string&gt;
     *  &lt;/in&gt;
     *  &lt;inout name="p2"&gt;
     *   &lt;long&gt;100&lt;/long&gt;
     *  &lt;/inout&gt;
     * &lt;/operation&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlOperation instance.
     */
    public IdlOperation(String xml) {
        this(XmlNode.getNode(xml));
    }

    /**
     * Constructs a IdlOperation instance from a interface repository id and an operation name.
     *
     * The constructed IdlOperation node has intizialized child node members which correspond
     * to the in and inout parameters for this operation.
     */
    public IdlOperation(String id, String name) {
        this();
        try {
            this.id = id;
            this.name = name;
            initParameters(id);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    IdlOperation(Node n) {
        this();
        try {
            id = XmlNode.getId(n);
            name = XmlNode.getName(n);

            initParameters(id);

            Node[] nodes = XmlNode.childElements(n);

            removeAllChildren();
            for (int i = 0; i < nodes.length; i++) {
                if (!names[i].equals(XmlNode.getName(nodes[i]))) {
                    throw new RuntimeException("Unknown parameter name: " + XmlNode.getName(nodes[i]) +  " for " + id + " " + name + " expected name: " + names[i]);
                }
                add(new IdlParameter(nodes[i]));
            }
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }

    /**
     * Intitializes parameter information data structures.
     * This is currently based on the presence of an extended helper class generated
     * by the DscGen idl compiler.
     * Alternativly, an interface repository server could be used.
     */
    private void initParameters(String id) {
        try {
            allDirections = XmlNode.getParameterDirections(id, name);
            allTypes = XmlNode.getParameterTypes(id, name);
            allNames = XmlNode.getParameterNames(id, name);
            exceptions = XmlNode.getParameterExceptions(id, name);

            for (int i = 0; i < allDirections.length; i++) {
                if (allDirections[i].equals("oneway")) {
                    isOneway = true;
                }
            }

            exceptionTypes = new org.omg.CORBA.TypeCode[exceptions.length];
            if (exceptions.length > 0) {
                isMutable = true;
                value = "no exception";
                for (int i = 0; i < exceptions.length; i++) {
                    exceptionTypes[i] = XmlNode.type(exceptions[i]);
                }
            }

            removeAllChildren();
            for (int i = 0; i < allNames.length; i++) {
                if (allDirections[i].startsWith("in")) {
                    add(new IdlParameter(allDirections[i], allNames[i], XmlNode.type(allTypes[i])));
                }
            }

            // The names and params array only have vales for  non void return, out and inout parameters
            int idx = getChildCount();
            names = new String[idx];
            typeNames = new String[idx];
            directions = new String[idx];
            params = new Parameter[idx];
            types = new org.omg.CORBA.TypeCode[idx];
            idx = 0;
            for (int i = 0; i < allNames.length; i++) {
                if (allDirections[i].startsWith("in")) {
                    params[idx] = new Parameter(allDirections[i], allTypes[i], allNames[i]);
                    names[idx] = allNames[i];
                    types[idx] = XmlNode.type(allTypes[i]);
                    typeNames[idx] = allTypes[i];
                    directions[idx] = allDirections[i];
                    idx++;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Return the list of parameter names for this operation.
     *
     * @return The list of parameter names.
     */
    public String[] getNames() {
        return names;
    }

    /**
     * Return the list of parameter type names for this operation.
     *
     * @return The list of parameter type names.
     */
    public String[] getTypes() {
        return typeNames;
    }

    /**
     * Return the list of parameter directions for this operation.
     *
     * @return The list of parameter directions.
     */
    public String[] getDirections() {
        return directions;
    }

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlOperation n, IdlWriter w) {
        w.write_start_operation(n.getName(), n.getId());
        for(int i = 0; i < n.getChildCount(); i++) {
            XmlNode.write((IdlNode)n.getChildAt(i), w);
        }
        w.write_end_operation();
    }

    public IdlReply invoke(org.omg.CORBA.ORB orb, org.omg.CORBA.Object target) {
        return dii_invoke(orb, target);
    }
        
    /**
     * Invokes the operation on a target CORBA object and returns an XmlReply object.
     *
     * @param orb The CORBA ORB reference.
     * @param target The target CORBA object on which to invoke the operation.
     *
     * @return An IdlReply object representing the result of the operation.
     */
    public IdlReply object_invoke(org.omg.CORBA.ORB orb, org.omg.CORBA.Object target) {
        org.omg.CORBA.portable.ObjectImpl objImpl = null;
        org.omg.CORBA.portable.OutputStream out = null;
        org.omg.CORBA.portable.InputStream in = null;        
        
        try {
//            Object tmp = (Object)(((org.omg.CORBA_2_3.ORB)orb).lookup_value_factory(org.omg.Components.ComponentPortDescriptionHelper.id()));

            objImpl = (org.omg.CORBA.portable.ObjectImpl)target;
            String oprName = name;
            Parameter[] inPar = null;
            Parameter[] outPar = null;
            org.omg.CORBA.Any[] inVal = null;
            org.omg.CORBA.Any[] outVal = null;
            
            out = objImpl._request(oprName, !isOneway);
            inPar = new Parameter[params.length];
            inVal = new org.omg.CORBA.Any[params.length];
            for (int i = 0; i < names.length; i++) {
                IdlParameter p = (IdlParameter)getChildAt(i);
                IdlNode n = p.getParameter();
                inPar[i] = new Parameter(directions[i], typeNames[i], names[i]);
                inVal[i] = n.toAny();             
            }

            for (int i = 0; i < inVal.length; i++) {
                inVal[i].write_value(out);
            }              
System.err.println("++++++++++++++ Before invoke: " + org.coach.tracing.service.ThreadContext.getEventCounter());
            try {
                in = objImpl._invoke(out);
            } catch(org.omg.CORBA.portable.ApplicationException aex) {
                in = aex.getInputStream();
                String aex_id = aex.getId();
                org.omg.CORBA.Any aex_value = orb.create_any();
                org.omg.CORBA.TypeCode etc = XmlNode.type(aex_id);
                aex_value.read_value(in, etc);
                IdlReply exReply = new IdlReply(id, name);
                exReply.setException(aex_value);
                return exReply;
            }                
System.err.println("++++++++++++++ After invoke: "  + org.coach.tracing.service.ThreadContext.getEventCounter());

            IdlReply reply = new IdlReply(id, name);
            if (!isOneway) {               
                outPar = reply.getParameterInfo();
                outVal = reply.readValues(in);               
            }
System.err.println("++++++++++++++ After reply: "  + org.coach.tracing.service.ThreadContext.getEventCounter());

            return reply;
        } catch (Throwable t) {
            t.printStackTrace();
            javax.swing.JOptionPane.showMessageDialog(null, t.toString());
        }

        return null;
    }

    /**
     * Invokes the operation on a target CORBA object and returns an XmlReply object.
     *
     * @param orb The CORBA ORB reference.
     * @param target The target CORBA object on which to invoke the operation.
     *
     * @return An IdlReply object representing the result of the operation.
     */
    public IdlReply dii_invoke(org.omg.CORBA.ORB orb, org.omg.CORBA.Object target) {
        org.omg.CORBA.Request request = target._request(name);

        IdlNode[] nodes = getParameters();
        // Vector to hold inout and out values for the request
        Vector v = new Vector();
        int ni = 0;
        for (int i = 0; i < allDirections.length; i++) {
            if (allDirections[i].equals("in")) {
                Any any = nodes[ni++].toAny();
                request.add_named_in_arg(allNames[i]).read_value(any.create_input_stream(), any.type());
            } else if (allDirections[i].equals("inout")) {
                Any any = nodes[ni++].toAny();
                Any inout = request.add_named_inout_arg(allNames[i]);
                inout.read_value(any.create_input_stream(), any.type());
                v.add(inout);
            } else if (allDirections[i].equals("out")) {
                Any out = request.add_named_out_arg(allNames[i]);
                out.type(XmlNode.type(allTypes[i]));
                v.add(out);
            }
        }

//        IdlReply reply = new IdlReply(id, name);

        request.set_return_type(XmlNode.type(allTypes[allTypes.length - 1]));
        for (int i = 0; i < exceptionTypes.length; i++) {
            request.exceptions().add(exceptionTypes[i]);
        }
        request.invoke();
        Exception ex = request.env().exception();
        
        IdlReply reply = new IdlReply(id, name);
        if (ex != null) {
            org.omg.CORBA.UnknownUserException uex = (org.omg.CORBA.UnknownUserException)ex;
            reply.setException(uex.except);
            return reply;
        }

        if (reply.hasReturn()) {
            v.add(request.result().value());
        }

        Any[] replyValues = new Any[v.size()];
        v.toArray(replyValues);

        reply.setValues(replyValues);

        return reply;
    }
}