/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

/**
 * Interface IdlWriter is used to write the content of an IdlNode derived type
 * as a formated string. For example class XmlWriter implements this interface to write
 * IdlNode values to XML format.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public interface IdlWriter {
    /**
     * Resets the output buffer of the IdlWriter.
     */
    public void clear();
    
    /**
     * Writes the value of a fixed IDL type to the IdlWriter.
     *
     * @param v The value of the fixed type.
     */
    public void write_fixed(String v);
    
    /**
     * Writes the value of a double IDL type to the IdlWriter.
     *
     * @param v The value of the double type.
     */
    public void write_double(String v);

    /**
     * Writes the value of a float IDL type to the IdlWriter.
     *
     * @param v The value of the float type.
     */
    public void write_float(String v);

    /**
     * Writes the value of a long double IDL type to the IdlWriter.
     *
     * @param v The value of the long double type.
     */
    public void write_longdouble(String v);

    /**
     * Writes the value of a unsigned long long IDL type to the IdlWriter.
     *
     * @param v The value of the unsigned long long type.
     */
    public void write_ulonglong(String v);

    /**
     * Writes the value of a long long IDL type to the IdlWriter.
     *
     * @param v The value of the long long type.
     */
    public void write_longlong(String v);

    /**
     * Writes the value of a unsigned long IDL type to the IdlWriter.
     *
     * @param v The value of the unsigned long type.
     */
    public void write_ulong(String v);

    /**
     * Writes the value of a long IDL type to the IdlWriter.
     *
     * @param v The value of the long type.
     */
    public void write_long(String v);

    /**
     * Writes the value of a unsigned short IDL type to the IdlWriter.
     *
     * @param v The value of the unsigned short type.
     */
    public void write_ushort(String v);

    /**
     * Writes the value of a short IDL type to the IdlWriter.
     *
     * @param v The value of the short type.
     */
    public void write_short(String v);

    /**
     * Writes the value of a octet IDL type to the IdlWriter.
     *
     * @param v The value of the octet type.
     */
    public void write_octet(String v);

    /**
     * Writes the value of a boolean IDL type to the IdlWriter.
     *
     * @param v The value of the boolean type.
     */
    public void write_boolean(String v);

    /**
     * Writes the value of a char IDL type to the IdlWriter.
     *
     * @param v The value of the char type.
     */
    public void write_char(String v);

    /**
     * Writes the value of a wchar IDL type to the IdlWriter.
     *
     * @param v The value of the wchar type.
     */
    public void write_wchar(String v);

    /**
     * Writes the value of a string IDL type to the IdlWriter.
     *
     * @param v The value of the string type.
     */
    public void write_string(String v);

    /**
     * Writes the value of a wstring IDL type to the IdlWriter.
     *
     * @param v The value of the wstring type.
     */
    public void write_wstring(String v);

    /**
     * Writes the value of a object IDL type to the IdlWriter.
     *
     * @param v The ior value of the object type.
     */
    public void write_Object(String v);

    /**
     * Writes the value of a sequence IDL type to the IdlWriter.
     *
     * A sequence is a nested type with zero or more elements.
     * A write_start_sequence operation must be followed by zero or more other write
     * operations which correspond with the sequence element types and must be closed by
     * a write_end_sequence operation.
     *
     * @param id The repository id of the sequence or an empty string for anonymous sequences.
     * @param length The number of sequence elements.
     */
    public void write_start_sequence(String id, int length);

    /**
     * Closes the nested sequence IDL type.
     */
    public void write_end_sequence();

    /**
     * Writes the value of a array IDL type to the IdlWriter.
     *
     * An array is a nested type with zero or more elements.
     * A write_start_array operation must be followed by zero or more other write
     * operations which correspond with the array element types and must be closed by
     * a write_end_array operation.
     *
     * @param id The repository id of the array or an empty string for anonymous arrays.
     * @param length The number of array elements.
     */
    public void write_start_array(String id, int length);

    /**
     * Closes the nested array IDL type.
     */
    public void write_end_array();

    /**
     * Writes the value of a struct IDL type to the IdlWriter.
     *
     * A struct is a nested type with one or more member elements.
     * A write_start_struct operation must be followed by one or more other write_start_member
     * and write_end_member operations for each struct member and must be closed by
     * a write_end_struct operation.
     *
     * @param id The repository id of the struct.
     */
    public void write_start_struct(String id);

    /**
     * Writes the name of a struct or union member IDL type to the IdlWriter.
     *
     * The write_start_member operation must be followed by a write operation
     * corresponding with the member type and must be closed by a write_end_member operation.
     *
     * @param name The name of the struct or union member.
     */
    public void write_start_member(String name);

    /**
     * Closes the nested struct or union member IDL type.
     */
    public void write_end_member();

    /**
     * Closes the nested struct or union IDL type.
     */
    public void write_end_struct();

    /**
     * Writes the value of a union IDL type to the IdlWriter.
     *
     * A union is a nested type with one member element.
     * A write_start_union operation must be followed by one write_union_tag operation and one write_start_member
     * and write_end_member operation for the union member and must be closed by
     * a write_end_union operation.
     *
     * @param id The repository id of the union.
     */
    public void write_start_union(String id);
    
    /**
     * Writes the tag value of a union IDL type to the IdlWriter.
     *
     * @param v The tag value of the union.
     */
    public void write_union_tag(String v);
    
    /**
     * Closes the nested union IDL type.
     */
    public void write_end_union();

    /**
     * Writes the value of a valuetype IDL type to the IdlWriter.
     *
     * A valuetype is a nested type with one or more member elements.
     * A write_start_value operation must be followed by one or more other write_start_member
     * and write_end_member operations for each struct member and must be closed by
     * a write_end_value operation.
     *
     * @param id The repository id of the struct.
     */
    public void write_start_value(String id);
    
    /**
     * Closes the nested valuetype IDL type.
     */
    public void write_end_value();

    /**
     * Writes the value of a enum IDL type to the IdlWriter.
     *
     * @param id The repository id of the enum.
     * @param v The value of the enum.
     */
    public void write_enum(String id, String v);

    /**
     * Writes the ior value of a interface IDL type to the IdlWriter.
     *
     * @param id The repository id of the interface.
     * @param v The ior value of the interface reference.
     */
    public void write_interface(String id, String ior);

    /**
     * Writes the ior value of a CCM component IDL type to the IdlWriter.
     *
     * @param id The repository id of the component.
     * @param v The ior value of the component reference.
     */
    public void write_component(String id, String ior);

    /**
     * Writes the name and type value of an IDL operation to the IdlWriter.
     *
     * The write_start_operation operation must be followed by a write operation corresponding
     * to the operation in and inout parameters and must be closed with a write_end_operation operation.
     *
     * @param name The name of the operation.
     * @param id The repository id of the interface to which the operation belongs.
     */
    public void write_start_operation(String name, String id);

    /**
     * Writes the name of an 'in' operation parameter.
     *
     * The write_start_in operation must be followed by a write operation corresponding
     * to the type of the parameter and must be closed with a write_end_inpar operation.
     *
     * @param name The name of the parameter.
     */
    public void write_start_inpar(String name);

    /**
     * Closes the nested write_start_inpar operation.
     */
    public void write_end_inpar();
    
    /**
     * Writes the name of an 'out' operation parameter.
     *
     * The write_start_out operation must be followed by a write operation corresponding
     * to the type of the parameter and must be closed with a write_end_outpar operation.
     *
     * @param name The name of the parameter.
     */
    public void write_start_outpar(String name);

    /**
     * Closes the nested write_start_outpar operation.
     */
    public void write_end_outpar();
    
    /**
     * Writes the name of an 'inout' operation parameter.
     *
     * The write_start_inout operation must be followed by a write operation corresponding
     * to the type of the parameter and must be closed with a write_end_inoutpar operation.
     *
     * @param name The name of the parameter.
     */
    public void write_start_inoutpar(String name);
        
    /**
     * Closes the nested write_start_inoutpar operation.
     */
    public void write_end_inoutpar();

    /**
     * Writes the return value of an operation.
     *
     * The write_start_return operation must be followed by a write operation corresponding
     * to the return type of the operation and must be closed with a write_end_return operation.
     */
    public void write_start_return();

    /**
     * Closes the nested write_start_return operation.
     */
    public void write_end_return();

    /**
     * Writes the exception value of an operation.
     *
     * The write_start_exception operation can be followed by a start_write_member operations corresponding
     * to the members of the exception and must be closed with a write_end_exception operation.
     *
     * @param id The repository id of the exception.
     */
    public void write_start_exception(String id);

    /**
     * Closes the nested write_start_exception operation.
     */
    public void write_end_exception();
    
    /**
     * Closes the nested write_start_operation operation.
     */
    public void write_end_operation();

    /**
     * Writes the name and type value of the reply of an IDL operation to the IdlWriter.
     *
     * The write_start_reply operation must be followed by a write operation corresponding
     * to the operation out and inout parameters and return value or exception value and
     * must be closed with a write_end_operation operation.
     *
     * @param name The name of the operation.
     * @param id The repository id of the interface to which the operation belongs.
     */
    public void write_start_reply(String name, String id);

    /**
     * Closes the nested write_start_reply operation.
     */
    public void write_end_reply();

    /**
     * Returns the content of the IdlWriter buffer as a String value.
     *
     * @return The content of the IdlWriter buffer as a String value.
     */
    public String toString();

    /**
     * Writes the value of a any IDL type to the IdlWriter.
     *
     * An any is a nested type with zero or one element.
     * A write_start_any operation must be followed by zero or one other write
     * operations which correspond with the content type of the any and must be closed by
     * a write_end_any operation.
     */
    public void write_start_any();

    /**
     * Closes the nested write_start_any operation.
     */
    public void write_end_any();
}