/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.generator.translator.idl2java.lib;

// Package dependencies.
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
import org.objectweb.openccm.ast.api.InterfaceDecl;
import org.objectweb.openccm.ast.api.OperationDecl;
import org.objectweb.openccm.ast.api.AttributeDecl;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.ParameterList;
import org.objectweb.openccm.ast.api.ExceptionList;

/**
 * This class translates some IDL declarations to their java mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class IDL_JavaTranslator
     extends org.objectweb.openccm.generator.translator.idl2java.lib.CommonTranslator
  implements org.objectweb.openccm.generator.translator.idl2java.api.IDL_JavaTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     **/
    public IDL_JavaTranslator()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.generator.translator.idl2java.api.IDL_JavaTranslator.
    //
    // ==================================================================

    /**
     * Map an IDL operation.
     *
     * @param op - The operation to map.
     *
     * @return The MethodObject binding to this operation.
     **/
    public MethodObject
    map_operation(OperationDecl op)
    {
        MethodObject method = null;

        method = new MethodObjectImpl();
        method.addComment("Implementation of the " + op.getAbsoluteName() + " operation.");
        method.setName( checkKeywords(op.getName()) );
        method.setReturnType( toJava(op.getType()) );
        map_parameters( method,
                        op.getParameterList() );
        map_exceptions( method,
                        op.getExceptionList() );
        return method;
    }

    /**
     * Map an IDL attribute to an accessor operation.
     *
     * @param att - The attribute to map.
     *
     * @return The MethodObject binding to this operation.
     **/
    public MethodObject
    map_attribute_accessor(AttributeDecl att)
    {
        MethodObject method = null;

        method = new MethodObjectImpl();
        method.addComment("Implementation of the " +
                           att.getAbsoluteName() + " attribute as accessor operation.");
        method.setName( checkKeywords(att.getName()) );
        method.setReturnType( toJava(att.getType()) );
        map_exceptions( method,
                        att.getExceptionList() );
        return method;
    }

    /**
     * Map an IDL attribute to a mutator operation.
     *
     * @param att - The attribute to map.
     *
     * @return The MethodObject binding to this operation.
     **/
    public MethodObject
    map_attribute_mutator(AttributeDecl att)
    {
        if (!att.isReadonly())
        {
            MethodObject method = null;
            ParameterObject param = null;

            method = new MethodObjectImpl();
            method.addComment("Implementation of the " +
                               att.getAbsoluteName() + " attribute as mutator operation.");
            method.setName( checkKeywords(att.getName()) );
            method.setReturnType("void");

            param = new ParameterObjectImpl();
            param.setName( checkKeywords(att.getName()) );
            param.setType( toJava(att.getType()) );
            method.addParameter(param);

            map_exceptions( method,
                            att.getExceptionList() );
            return method;
        }
        return null;
    }

    /**
     * Map a parameter list for a method.
     *
     * @param method - Add parameters mapping to this method.
     * @param list - The parameter list to map.
     **/
    public void
    map_parameters(MethodObject method,
                   ParameterList list)
    {
        org.objectweb.openccm.ast.api.Parameter[] params = null;
        ParameterObject param = null;

        params = list.getParameters();
        for (int i=0; i<params.length; i++)
        {
            param = new ParameterObjectImpl();
            param.setName( checkKeywords(params[i].getName()) );
            param.setType( toJava(params[i].getType(), params[i].getMode()) );
            method.addParameter(param);
        }
    }

    /**
     * Map an exeception list for a method.
     *
     * @param method - Add exceptions mapping to this method.
     * @param list - The exception list to map.
     **/
    public void
    map_exceptions(MethodObject method,
                   ExceptionList list)
    {
        org.objectweb.openccm.ast.api.ExceptionDecl[] excs = null;

        excs = list.getExceptions();
        for (int i=0; i<excs.length; i++)
        {
            method.addException( getAbsoluteName(excs[i]) );
        }
    }

    /**
     * Generate operations mapping for an interface.
     *
     * @param clazz - Add methods to the segment class.
     **/
    public void
    mapInterfaceOps(ClassObject clazz,
                    InterfaceDecl itf)
    {
        Declaration[] decls = null;
        MethodObject method = null;

        decls = itf.getContents(false, DeclarationKind.dk_operation);
        for (int i=0; i<decls.length; i++)
        {
            method = map_operation( (OperationDecl)decls[i] );
            method.getImpl().setMacro("BUSINESS_OP");
            method.getImpl().addContextValue("translator", this);
            method.getImpl().addContextValue("obj", (OperationDecl)decls[i]);
            clazz.addMethod(method);
        }
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
